#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2013  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Utils: import archaelogical operation from a DBF file
"""

from __future__ import unicode_literals

import datetime
import dbf
import re

from django.contrib.auth.models import User
from django.db import transaction

from archaeological_operations.import_from_csv import parse_operationtype, \
                    parse_multivalue, parse_person as _parse_person, parse_date,\
    Column, parse_patriarche_operationtype, parse_ope_name
from archaeological_operations.models import Operation, OperationType, Period, \
                                             AdministrativeAct, ActType

def parse_person(surname, name, owner):
    return _parse_person(surname, name, None, owner)

def parse_ha(value):
    try:
        val = float(value)
    except ValueError:
        val = 0
    return val * 10000

period_types = {
    '':'not_yet_documented',
    'IND':'indetermined',
    'CON': 'contemporan',
    'MOD': 'modern',
    'REC': 'recent_times',
    'BMA': 'low_middle_age',
    'MAC': 'classic_middle_age',
    'HMA': 'high_middle_age',
    'MA' : 'middle_age',
    'MED': 'middle_age',
    'BAS': 'low_empire',
    'HAU': 'high-empire',
    'NRE': 'republic',
    'GAL': 'gallo-roman',
    'FE2': 'second_iron_age',
    'FE1': 'first_iron_age',
    'BRF': 'final_bronze_age',
    'BRM': 'middle_bronze_age',
    'BRA': 'old_bronze_age',
    'FER': 'iron_age',
    'BRO': 'bronze_age',
    'PRO': 'protohistory',
    'NEF': 'final_neolithic',
    'NER': 'recent_neolithic',
    'NEM': 'middle_neolithic',
    'NEA': 'old_neolithic',
    'NEO': 'neolithic',
    'MER': 'recent_mesolithic',
    'MEM': 'middle_mesolithic',
    'MEA': 'old_mesolithic',
    'MES': 'mesolithic',
    'PAF': 'final_paleolithic',
    'PAS': 'late_paleolithic',
    'PAM': 'middle_paleolithic',
    'PAA': 'ancien_paleolithic',
    'PAL': 'paleolithic'
}

_CACHED_PERIOD_TYPES = {}

def _prepare_period_types():
    for k in period_types.keys():
        _CACHED_PERIOD_TYPES[k] = Period.objects.get(txt_idx=period_types[k])

_period_re_filter = re.compile('^EUR')
_period2_re_filter = re.compile('-*$')

def parse_period(value):
    value = _period_re_filter.sub('', value)
    value = _period2_re_filter.sub('', value)
    if value not in _CACHED_PERIOD_TYPES.keys():
        value = ''
    return _CACHED_PERIOD_TYPES[value]

PATRIARCHE_DBF_OPE_COLS = [
    Column(('operation_type',), 'parse_patriarche_operationtype'),
    Column(('common_name',), 'parse_ope_name'),
    [],
    Column(('in_charge',), 'parse_person', [2]),
    [],  # 'etat'
    Column(('comment',), unicode),  #'adresse'
    [],  #'origine C(3)'
    Column(('periods',), 'parse_period', multi=True),
    [],  #'programme C(254)'
    [],  # 'rattach_pc C(254)'
    [],  # 'code_dossi N(8,0)'
    Column(('administrative_act', 'ref_sra'), unicode),
    Column(('administrative_act', 'signature_date'), parse_date),
    Column(('start_date',), parse_date),
    Column(('end_date',), parse_date),
    Column(('year',), int, []),
    [],  # 'identifica C(254)'
    Column(('code_patriarche',), int),
    [],  # 'x_degre N(16,6)'),
    [],  # 'y_degre N(16,6)'),
    [],  # 'x_saisi C(12)'),
    [],  # 'y_saisi C(12)'),
    [],  # 'georeferen C(3)'),
    [],  # 'geometrie C(3)'),
    Column(('surface',), parse_ha)
]

DBF_OPE_COLS = PATRIARCHE_DBF_OPE_COLS

def import_from_dbf(filename, update=False, col_defs=DBF_OPE_COLS,
                    person=None, stdout=None):
    """
    Import from a DBF file.
    Return number of operation treated and errors.
    """
    try:
        table =  dbf.Table(filename)
    except (dbf.DbfError, TypeError):
        return 0, ["Incorrect DBF file."]

    new_ops, errors = import_operations_dbf(table, col_defs=col_defs,
                                    update=update, person=person, stdout=stdout)
    return new_ops, errors

ERROR_LBLS = {'missing_ope_type':'* Missing operation type: ',
              'missing_patriarche_code':'* Missing patriarche code: '}

@transaction.commit_manually
def import_operations_dbf(values, col_defs=DBF_OPE_COLS, update=False,
                          person=None, stdout=None):
    default_person = person or User.objects.order_by('pk').all()[0]
    # key : (class, default, reverse)
    key_classes = {
    'administrative_act':(AdministrativeAct, {'history_modifier':default_person,
                           'act_type':ActType.objects.get(
                                      txt_idx='excavation_order')}, 'operation'),
    }
    _prepare_ope_types()
    _prepare_period_types()

    ope_default = {'history_modifier':default_person}
    current_import = []
    new_ops = 0
    errors_nb = {}
    for error in ERROR_LBLS.keys():
        errors_nb[error] = 0
    error_ope, error_reversed, error_multis = [], [], []
    multi_keys = set([column.col_models[0]
                      for column in col_defs if column and column.multi])
    for line_idx, vals in enumerate(values):
        if stdout:
            stdout.write("\r* line %d" % (line_idx))
        args = {}
        for col_idx, val in enumerate(vals):
            if len(col_defs) <= col_idx or not col_defs[col_idx]:
                continue
            col_def = col_defs[col_idx]
            attrs, typ = col_def.col_models, col_def.format
            extra_cols = col_def.associated_cols
            if not callable(typ):
                typ = globals()[typ]
            c_args = args
            for attr in attrs:
                if attr not in c_args:
                    c_args[attr] = {}
                c_args = c_args[attr]
            if not extra_cols:
                try:
                    v = typ(val)
                except TypeError:
                    v = None
            else:
                arguments = [vals[col_number] for col_number in extra_cols]
                if not [arg for arg in arguments if arg]:
                    continue
                arguments += [default_person]
                v = typ(val, *arguments)
            if len(attrs) == 1:
                args[attrs[0]] = v
            elif len(attrs) == 2:
                args[attrs[0]][attrs[1]] = v
            elif len(attrs) == 3:
                args[attrs[0]][attrs[1]][attrs[2]] = v
        # manage exploded dates
        for k in args.keys():
            if '__year' in k:
                key = k[:-len('__year')]
                try:
                    v = datetime.datetime(args[k], args[key+'__month'],
                                          args[key+'__day'])
                    args[key] = v
                except:
                    pass
                args.pop(k)
                args.pop(key+'__month')
                args.pop(key+'__day')
        reversed_items, multis = [], []
        for k in args.keys():
            if k in key_classes:
                cls, default, reverse = key_classes[k]
                default.update(args[k])
                if reverse:
                    reversed_items.append((cls, default, reverse))
                    args.pop(k)
                    continue
                try:
                    obj = cls.objects.get(**default)
                except:
                    obj = cls.objects.create(**default)
                    obj.save()
                transaction.commit()
                args[k] = obj
            elif type(args[k]) == list or k in multi_keys:
                multis.append((k, args[k]))
                args.pop(k)
        op = None
        if not update and not args.get('operation_type'):
            errors_nb['missing_ope_type'] += 1
            continue
        try:
            op = Operation.objects.get(code_patriarche=args['code_patriarche'])
            if not update and op.pk not in current_import:
                errors_nb['already_available_patriarche_code'] += 1
                continue
        except:
            pass
        # check
        if not args.get('year') and args.get('start_date'):
            args['year'] = args['start_date'].year
        updated = False
        # creation
        if not op:
            args.update(ope_default)
            for k in args.keys():
                if not args[k]:
                    args.pop(k)
            op = Operation.objects.create(**args)
            new_ops += 1
            transaction.commit()
        else: # mise à jour
            try:
                for k in args:
                    if not args[k]:
                        args[k] = None
                    #if getattr(op, k):
                    #    continue
                    setattr(op, k, args[k])
                op.save()
            except:
                transaction.rollback()
                continue
            transaction.commit()
            updated = True
        try:
            for cls, default, reverse in reversed_items:
                default[reverse] = op
                it = cls(**default).save()
        except:
            transaction.rollback()
            current_import.append(op.pk)
            error_reversed.append((line_idx, reversed_items))
            continue
        transaction.commit()
        try:
            for k, vals in multis:
                if op.pk not in current_import and updated:
                    getattr(op, k).clear()
                if type(vals) not in (list, tuple):
                    vals = [vals]
                for v in vals:
                    getattr(op, k).add(v)
                    op.save()
        except:
            transaction.rollback()
            current_import.append(op.pk)
            error_multis.append((line_idx, multis))
            continue
        transaction.commit()
        current_import.append(op.pk)

    errors = []
    for error_key in errors_nb:
        nb_error = errors_nb[error_key]
        if nb_error:
            errors.append(ERROR_LBLS[error_key] + str(nb_error))
    if error_ope:
        error = "Error while recording these operations:\n"
        for line_idx, args in error_ope:
            error += "line: " + str(line_idx) + " args: " + str(args) + '\n'
        errors.append(error)
    if error_multis:
        error = "Error while recording these multiples items attached to "\
                "operation:"
        for line_idx, args in error_multis:
            error += "line: " + str(line_idx) + " args: " + str(args) + '\n'
        errors.append(error)
    if error_reversed:
        error = "Error while recording these items that depend to operation:"
        for line_idx, args in error_reversed:
            error += "line: " + str(line_idx) + " args: " + str(args) + '\n'
        errors.append(error)
    return new_ops, errors




import os
import shutil
import sys
from zipfile import ZipFile, ZIP_DEFLATED

rpl_lst = [
    ('adminact_associated_file_general_contractor_attached_to_name',
     'adminact_associated_file_corporation_general_contractor_name'),
    ('adminact_associated_file_general_contractor_'
     'attached_to_address',
     'adminact_associated_file_corporation_general_contractor_'
     'address'),
    ('adminact_associated_file_general_contractor_'
     'address_complement',
     'adminact_associated_file_corporation_general_contractor_'
     'address_complement '),
    ('adminact_associated_file_general_contractor_'
     'attached_to_postal_code',
     'adminact_associated_file_corporation_general_contractor_'
     'postal_code '),
    ('adminact_associated_file_general_contractor_attached_to_town',
     'adminact_associated_file_corporation_general_contractor_town',
     ),
    ('adminact_associated_file_address',
     'adminact_associated_file_get_locality',
     )
]

context = dict(rpl_lst)


def value_replace(content):
    value = content
    modified = False
    for key in context:
        if key in value:
            modified = True
            value = value.replace(key, context[key])
    return value, modified


def replace(directory, infile):
    print("Processing {}".format(infile))
    outfile = "PREPROCESS--" + infile
    infile = directory + os.sep + infile
    outfile = directory + os.sep + outfile

    inzip = ZipFile(infile, 'r', ZIP_DEFLATED)
    outzip = ZipFile(outfile, 'w', ZIP_DEFLATED)

    values = {}
    idx = 0
    for xml_file in ('content.xml', 'styles.xml'):
        content = inzip.read(xml_file)
        values[xml_file], modified = value_replace(content)
        if modified:
            idx += 1

    for f in inzip.infolist():
        if f.filename in values:
            outzip.writestr(f.filename, values[f.filename])
        else:
            outzip.writestr(f, inzip.read(f.filename))

    inzip.close()
    outzip.close()
    # replace original by PREPROCESS
    shutil.move(outfile, infile)
    return idx

directory = sys.argv[-1]
idx = 0


for fle in os.listdir(directory):
    if fle.endswith('.odt'):
        idx += replace(directory, fle)

print("{} modifications".format(idx))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2013  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

DELIMITER = ";"
QUOTECHAR = '"'

import datetime
import re

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.template.defaultfilters import slugify

from django.contrib.auth.models import User
from ishtar_common.models import Town, Person, PersonType, OrganizationType, \
    Organization, SourceType
from archaeological_files.models import PermitType
from archaeological_operations.models import OperationType, Period, \
    ActType


def get_default_person():
    return User.objects.order_by('pk').all()[0]


def _get_parse_string(trunc_number=None):
    def parse_string(value):
        value = value.strip()
        if value == '#EMPTY':
            value = ''
        value = value.replace('  ', ' ')
        if trunc_number:
            value = value[:trunc_number]
        return value
    return parse_string

parse_string = _get_parse_string()


def parse_multivalue(value):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1 \2', value)
    s1 = re.sub('([a-z0-9])([A-Z])', r'\1 \2', s1)
    return re.sub('([0-9])([a-z])', r'\1 \2', s1)

ope_types = {}


def _init_ope_types():
    for k in settings.ISHTAR_OPE_TYPES.keys():
        ot, created = OperationType.objects.get_or_create(
            txt_idx=settings.ISHTAR_OPE_TYPES[k][0],
            defaults={'label': settings.ISHTAR_OPE_TYPES[k][1],
                      'preventive': k[0] == u'préventive'})
        ope_types[k] = ot


def parse_operationtype(value, preventive, owner):
    value = (preventive.strip(), value.strip())
    if not ope_types:
        _init_ope_types()
    if value not in ope_types:
        return None
    return ope_types[value]

periods = {}
periods_keys = []


def _init_period():
    for k in settings.ISHTAR_PERIODS.keys():
        periods[k] = Period.objects.get(txt_idx=settings.ISHTAR_PERIODS[k])
    periods_keys = periods.keys()
    periods_keys.sort(key=len)
    periods_keys.reverse()
    for k in settings.ISHTAR_PERIODS.keys():
        period = Period.objects.get(txt_idx=settings.ISHTAR_PERIODS[k])
        slug = slugify(period.label)
        period_names[slug] = period
        for k in REPLACED_PERIOD_DCT.keys():
            if k in slug:
                period_names[slug.replace(k, REPLACED_PERIOD_DCT[k])] = period
    period_names_keys = period_names.keys()
    period_names_keys.sort(key=len)
    period_names_keys.reverse()


def parse_period(value):
    value = parse_string(value)
    value = value[3:] if value.startswith('EUR') else value
    while value.endswith('-'):
        value = value[:-1]
    value = value[3:] if value.startswith('EUR') else value
    if not periods:
        _init_period()
    if not value:
        return [periods[u'']]
    period, old_val = [], u''
    while value and old_val != value:
        old_val = value
        for k in periods_keys:
            if value.startswith(k):
                period.append(periods[k])
                value = value[len(k):]
                break
    return period

_REPLACED_PERIOD = [('deuxieme', 'second')]
_REPLACED_PERIOD += [(y, x) for x, y in _REPLACED_PERIOD]
REPLACED_PERIOD_DCT = dict(_REPLACED_PERIOD)

period_names = {}
period_names_keys = {}


def parse_period_name(value):
    if not period_names:
        _init_period()
    value = parse_string(value)
    if not value:
        return [period_names[u'']]
    period, old_val = [], u''
    value = slugify(value)
    while value and old_val != value:
        old_val = value
        for k in period_names_keys:
            if value.startswith(k):
                period.append(period_names[k])
                value = value[len(k):]
                break
    return period

_CACHED_PERMIT_TYPES = {}


def _init_permit_type():
    for k in settings.ISHTAR_PERMIT_TYPES:
        txt_idx, label = settings.ISHTAR_PERMIT_TYPES[k]
        permit_type, created = PermitType.objects.get_or_create(
            txt_idx=txt_idx, defaults={'label': label,
                                       'available': True})
        _CACHED_PERMIT_TYPES[k] = permit_type


def parse_permittype(value):
    value = parse_string(value).lower()
    if not _CACHED_PERMIT_TYPES:
        _init_permit_type()
    if value not in _CACHED_PERMIT_TYPES:
        if "" not in _CACHED_PERMIT_TYPES:
            return
        value = ""
    return _CACHED_PERMIT_TYPES[value]

_CACHED_ADMIN_ACT_TYPES = {}


def parse_admin_act_typ(value, code, owner):
    value = parse_string(value).lower()
    code = parse_string(code).lower()
    if not value or not code:
        return
    if code not in _CACHED_ADMIN_ACT_TYPES:
        act_type, created = ActType.objects.get_or_create(
            txt_idx=code, defaults={'label': value})
        _CACHED_ADMIN_ACT_TYPES[code] = act_type
    return _CACHED_ADMIN_ACT_TYPES[code]


def parse_fileref(value):
    value = parse_string(value).split('/')[0]
    value = value.split('.')[0]
    match = re.search('[0-9].[0-9]*', value)
    if not match:
        return None
    return int(match.group())


def parse_orga(value, alternate_value, owner):
    value = parse_string(value)
    if not value:
        value = parse_string(alternate_value)
        if not value:
            return
    q = Organization.objects.filter(name__iexact=value)
    if q.count():
        return q.all()[0]
    try:
        organization_type = OrganizationType.objects.get(label__iexact=value)
    except ObjectDoesNotExist:
        organization_type = OrganizationType.objects.get(txt_idx='undefined')
    orga = Organization.objects.create(name=value,
                                       organization_type=organization_type,
                                       history_modifier=owner)
    return orga


def parse_bool(value):
    value = parse_string(value)
    if value.lower() in ('yes', 'oui'):
        value = True
    elif value.lower() in ('no', 'non'):
        value = False
    else:
        value = None
    return value


def parse_date(value):
    value = parse_string(value).split(' ')[0]
    try:
        return datetime.datetime.strptime(value, '%d/%m/%Y')
    except:
        return None


def parse_yearref(value):
    value = parse_string(value).split('.')[0]
    match = re.search('[0-9].[0-9]*', value)
    if not match:
        return None
    return int(match.group())


def parse_surface(value):
    value = parse_string(value)
    value = value.replace(',', '.')
    try:
        # hectare en metre carrés
        value = float(value) * 10000
        if value:
            return value
        return None
    except:
        return None


def parse_year(value):
    value = parse_string(value)
    try:
        yr = int(value)
    except:
        return None
    if yr < 1900 or yr > 2100:
        return None
    return yr


def parse_trunc_patriarche(value):
    value = parse_string(value)
    if not value:
        return
    value = value.replace(' ', '')
    try:
        int(value)
    except:
        return
    return '18' + unicode(value)


def parse_operation_code(value):
    value = parse_string(value)
    code = value.split('.')[-1]
    try:
        return int(code)
    except:
        return


def parse_title(value):
    value = parse_string(value)
    if not value:
        return
    return value.title()


def parse_name_surname(value, owner):
    value = parse_string(value)
    items = value.split(' ')
    name = items[0]
    surname = ""
    if len(items) > 1:
        name = " ".join(items[:-1])
        surname = items[-1]
    values = {"surname": parse_title(surname)[:30],
              "name": parse_title(name)[:30]}
    if not values['surname'] and not values['name']:
        return
    q = Person.objects.filter(**values)
    if q.count():
        return q.all()[0]
    else:
        defaults = {'history_modifier': owner,
                    'title': ''}
        defaults.update(values)
        p = Person.objects.create(**defaults)
        p.person_types.add(PersonType.objects.get(
            txt_idx='head_scientist'))
        return p


def parse_person(surname, name, old_ref, owner):
    values = {"surname": parse_title(surname),
              "name": parse_title(name)}
    if not values['surname'] and not values['name']:
        return
    q = Person.objects.filter(**values)
    if q.count():
        return q.all()[0]
    else:
        defaults = {'history_modifier': owner,
                    'title': ''}
        defaults.update(values)
        p = Person.objects.create(**defaults)
        p.person_types.add(PersonType.objects.get(
            txt_idx='head_scientist'))
        return p


def parse_comment_addr_nature(nature, addr, owner):
    addr = parse_string(addr)
    nature = parse_string(nature)
    comments = []
    if nature:
        comments += [u"Aménagement :", nature]
    if addr:
        comments += [u"Adresse :", addr]
    if not comments:
        return ""
    return u"\n".join(comments)

# si pas de start date : premier janvier de year

ope_types = {
    'AET': ('other_study',
            'Autre étude', True),
    'APP': ('assistance_preparation_help',
            'Aide à la préparation de publication', True),
    'DOC': ('documents_study',
            'Étude documentaire', True),
    'EV': ('evaluation',
           "Fouille d'évaluation", True),
    'FOU': ('ancient_excavation',
            "Fouille ancienne", True),
    'FP': ('prog_excavation',
           "Fouille programmée", False),
    'MH': ('building_study', "Fouille avant MH", True),
    'OPD': ('arch_diagnostic',
            "Diagnostic archéologique", True),
    'PAN': ('analysis_program',
            "Programme d'analyses", False),
    'PCR': ('collective_research_project',
            "Projet collectif de recherche", False),
    'PMS': ('specialized_eqp_prospection',
            "Prospection avec matériel spécialisé", False),
    'PRD': ('diachronic_prospection',
            "Prospection diachronique", False),
    'PI': ('diachronic_prospection',
           "Prospection diachronique", False),
    'PRM': ('metal_detector_prospection',
            "Prospection détecteur de métaux", False),
    'PRT': ('thematic_prospection',
            "Prospection thématique", False),
    'PT': ('thematic_prospection',
           "Prospection thématique", False),
    'RAR': ('cave_art_record',
            "Relevé d'art rupestre", False),
    'SD': ('sampling_research',
           "Sondage", False),
    'SP': ('prev_excavation',
           "Fouille préventive", True),
    'SU': ('emergency_excavation',
           "Fouille préventive d'urgence", True),
}

_CACHED_OPE_TYPES = {}


def _prepare_ope_types():
    for k in ope_types.keys():
        txt_idx, label, preventive = ope_types[k]
        ot, created = OperationType.objects.get_or_create(
            txt_idx=txt_idx, defaults={'label': label,
                                       'preventive': preventive})
        if k not in _CACHED_OPE_TYPES.keys():
            _CACHED_OPE_TYPES[k] = ot


def parse_patriarche_operationtype(value):
    if value not in _CACHED_OPE_TYPES.keys():
        return None
    return _CACHED_OPE_TYPES[value]

_dpt_re_filter = re.compile('^\([0-9]*\) ')


def parse_ope_name(value):
    if not value:
        return ''
    value = value.strip()
    if value.lower() == 'null':
        return ''
    value = _dpt_re_filter.sub('', value)
    return value


def parse_ha(value):
    value = parse_string(value)
    try:
        value = float(value) * 10000
    except:
        value = None
    return value


def parse_rapp_index(value):
    value = parse_string(value)
    items = re.findall(r'[0-9]+$', value)
    if items:
        return int(items[-1])

_CACHED_DOC_TYPES = {}


def parse_doc_types(value):
    value = parse_string(value)
    if value not in _CACHED_DOC_TYPES:
        if value not in settings.ISHTAR_DOC_TYPES:
            return
        _CACHED_DOC_TYPES[value], created = SourceType.objects.get_or_create(
            txt_idx=value,
            defaults={"label": settings.ISHTAR_DOC_TYPES[value]})
    return _CACHED_DOC_TYPES[value]


def parse_insee(value):
    value = parse_string(value)
    values = []
    while len(value) > 4:
        values.append(value[:5])
        value = value[5:]
    towns = []
    for value in values:
        try:
            town = Town.objects.get(numero_insee=value)
            towns.append(town)
        except:
            # sys.stderr.write('Numero INSEE : %s non existant en base'
            # % value)
            continue
    return towns


PARCEL_YEAR_REGEXP = re.compile(r"^([0-9]{4})[ :]+")
PARCEL_SECTION_REGEXP = re.compile(
    ur"(?: )*(?:[Ss]ection(?:s)?)?(?: )*([A-Z][A-Z0-9]{0,3})[ :]*"
    ur"((?:(?: |;|,|[Pp]arcelle(?:s)?|n°|et|à|to)*[0-9]+[p]?)+)")
PARCEL_NB_RANGE_REGEXP = re.compile(ur'([0-9]+[p]?) (?:à|to) ([0-9]+[p]?)')
PARCEL_NB_REGEXP = re.compile(
    ur'(?: |;|,|[Pp]arcelle(?:s)?|n°|et|à|to)*([0-9]+[p]?)')


def parse_parcels(parcel_str, insee_code=None, owner=None):
    parcels, town = [], None
    if insee_code:
        town = parse_insee(insee_code)
        # manage only one town at a time
        assert len(town) < 2
        if not town:
            return parcels
        town = town[0]
    m = PARCEL_YEAR_REGEXP.match(parcel_str)
    year = None
    if m:
        year = m.groups()[0]
        parcel_str = parcel_str[m.span()[1]:]
    for parcel in PARCEL_SECTION_REGEXP.findall(parcel_str):
        sector, nums = parcel[0], parcel[1]
        for num in PARCEL_NB_REGEXP.findall(nums):
            if len(unicode(num)) > 6:
                continue
            dct = {'year': year, 'section': sector, 'parcel_number': num}
            if town:
                dct['town'] = town
            if owner:
                dct['history_modifier'] = owner
            parcels.append(dct)
        for parcel_ranges in PARCEL_NB_RANGE_REGEXP.findall(nums):
            lower_range, higher_range = parcel_ranges
            try:
                # the lower range itself has been already kept
                lower_range = int(lower_range) + 1
                higher_range = int(higher_range)
            except ValueError:
                continue
            for num in xrange(lower_range, higher_range):
                dct = {'year': year, 'section': sector,
                       'parcel_number': unicode(num)}
                if town:
                    dct['town'] = town
                if owner:
                    dct['history_modifier'] = owner
                parcels.append(dct)
    return parcels



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Files forms definitions
"""
import datetime

from django import forms
from django.conf import settings
from django.core import validators
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from ishtar_common.models import Person, PersonType, Organization, \
    OrganizationType, valid_id, Department
from archaeological_operations.models import ActType, AdministrativeAct, \
    OperationType
import models
from ishtar_common.forms import FinalForm, get_now, reverse_lazy, TableSelect, \
    ManageOldType
from ishtar_common.forms_common import get_town_field
from archaeological_operations.forms import AdministrativeActOpeForm, \
    AdministrativeActOpeFormSelection, \
    ParcelField, SLICING, HEAD_SCIENTIST, SRA_AGENT, AdministrativeActModifForm
from ishtar_common import widgets

GENERAL_CONTRACTOR, created = PersonType.objects.get_or_create(
    txt_idx='general_contractor')

GENERAL_CONTRACTOR_ORGA, created = OrganizationType.objects.get_or_create(
    txt_idx='general_contractor')


class FileSelect(TableSelect):
    year = forms.IntegerField(label=_("Year"))
    numeric_reference = forms.IntegerField(label=_("Numeric reference"))
    internal_reference = forms.CharField(max_length=200,
                                         label=_("Other reference"))
    towns = get_town_field()
    parcel = ParcelField(label=_("Parcel (section/number/public domain)"))
    if settings.ISHTAR_DPTS:
        towns__numero_insee__startswith = forms.ChoiceField(
            label=_(u"Department"), choices=[])
    name = forms.CharField(label=_(u"File name"), max_length=200)
    file_type = forms.ChoiceField(label=_("File type"), choices=[])
    end_date = forms.NullBooleanField(label=_(u"Is active?"))
    saisine_type = forms.ChoiceField(label=_("Saisine type"), choices=[])
    permit_type = forms.ChoiceField(label=_("Permit type"), choices=[])
    permit_reference = forms.CharField(max_length=200,
                                       label=_("Permit reference"))
    comment = forms.CharField(label=_(u"Comment"), max_length=500)
    in_charge = forms.IntegerField(
        label=_(u"In charge"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-person',
                         args=[SRA_AGENT.pk]),
            associated_model=Person),
        validators=[valid_id(Person)])
    general_contractor = forms.IntegerField(
        label=_(u"General contractor"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-person',
                         args=[GENERAL_CONTRACTOR.pk]),
            associated_model=Person),
        validators=[valid_id(Person)])
    general_contractor__attached_to = forms.IntegerField(
        label=_(u"Organization of general contractor"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-organization',
                         args=[GENERAL_CONTRACTOR_ORGA.pk]),
            associated_model=Organization),
        validators=[valid_id(Organization)])
    history_creator = forms.IntegerField(
        label=_(u"Created by"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-person',
                         args=['0', 'user']),
            associated_model=Person),
        validators=[valid_id(Person)])
    history_modifier = forms.IntegerField(
        label=_(u"Modified by"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-person',
                         args=['0', 'user']),
            associated_model=Person),
        validators=[valid_id(Person)])

    def __init__(self, *args, **kwargs):
        super(FileSelect, self).__init__(*args, **kwargs)
        self.fields['saisine_type'].choices = \
            models.SaisineType.get_types()
        self.fields['saisine_type'].help_text = models.SaisineType.get_help()
        self.fields['permit_type'].choices = models.PermitType.get_types()
        self.fields['permit_type'].help_text = models.PermitType.get_help()
        self.fields['file_type'].choices = models.FileType.get_types()
        self.fields['file_type'].help_text = models.FileType.get_help()
        if settings.ISHTAR_DPTS:
            k = 'towns__numero_insee__startswith'
            self.fields[k].choices = [
                ('', '--')] + list(settings.ISHTAR_DPTS)

    def get_input_ids(self):
        ids = super(FileSelect, self).get_input_ids()
        ids.pop(ids.index('parcel'))
        ids.append('parcel_0')
        ids.append('parcel_1')
        ids.append('parcel_2')
        return ids


class FileFormSelection(forms.Form):
    form_label = _("Archaeological file search")
    associated_models = {'pk': models.File}
    currents = {'pk': models.File}
    pk = forms.IntegerField(
        label="", required=False,
        widget=widgets.JQueryJqGrid(
            reverse_lazy('get-file'),
            FileSelect, models.File,
            source_full=reverse_lazy('get-file-full')),
        validators=[valid_id(models.File)])

    def clean(self):
        cleaned_data = self.cleaned_data
        if 'pk' not in cleaned_data or not cleaned_data['pk']:
            raise forms.ValidationError(_(u"You should select a file."))
        return cleaned_data

DATE_SOURCE = (('creation', _(u"Creation date")),
               ("reception", _(u"Reception date")))


class DashboardForm(forms.Form):
    slicing = forms.ChoiceField(
        label=_("Slicing"), choices=SLICING, required=False)
    department_detail = forms.BooleanField(
        label=_("Department detail"), required=False)
    date_source = forms.ChoiceField(
        label=_("Date get from"), choices=DATE_SOURCE, required=False)
    file_type = forms.ChoiceField(
        label=_("File type"), choices=[], required=False)
    saisine_type = forms.ChoiceField(
        label=_("Saisine type"), choices=[], required=False)
    after = forms.DateField(
        label=_(u"Date after"), widget=widgets.JQueryDate, required=False)
    before = forms.DateField(
        label=_(u"Date before"), widget=widgets.JQueryDate, required=False)

    def __init__(self, *args, **kwargs):
        if 'prefix' not in kwargs:
            kwargs['prefix'] = 'files'
        super(DashboardForm, self).__init__(*args, **kwargs)
        self.fields['saisine_type'].choices = models.SaisineType.get_types()
        self.fields['file_type'].choices = models.FileType.get_types()

    def get_show_detail(self):
        return hasattr(self, 'cleaned_data') and \
            self.cleaned_data.get('department_detail')

    def get_date_source(self):
        date_source = 'creation'
        if hasattr(self, 'cleaned_data') and \
           self.cleaned_data.get('date_source'):
            date_source = self.cleaned_data['date_source']
        return date_source

    def get_filter(self):
        if not hasattr(self, 'cleaned_data') or not self.cleaned_data:
            return {}
        date_source = self.get_date_source()
        fltr = {}
        if self.cleaned_data.get('saisine_type'):
            fltr['saisine_type_id'] = self.cleaned_data['saisine_type']
        if self.cleaned_data.get('file_type'):
            fltr['file_type_id'] = self.cleaned_data['file_type']
        if self.cleaned_data.get('after'):
            fltr[date_source + '_date__gte'] = self.cleaned_data['after']
        if self.cleaned_data.get('before'):
            fltr[date_source + '_date__lte'] = self.cleaned_data['before']
        return fltr


class FileFormGeneral(ManageOldType, forms.Form):
    form_label = _("General")
    associated_models = {'in_charge': Person,
                         'related_file': models.File,
                         'file_type': models.FileType}
    in_charge = forms.IntegerField(
        label=_("Person in charge"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-person', args=[SRA_AGENT.pk]),
            limit={'person_types': [SRA_AGENT.pk]},
            associated_model=Person, new=True),
        validators=[valid_id(Person)])
    year = forms.IntegerField(label=_("Year"),
                              initial=lambda: datetime.datetime.now().year,
                              validators=[validators.MinValueValidator(1900),
                                          validators.MaxValueValidator(2100)])
    numeric_reference = forms.IntegerField(
        label=_("Numeric reference"), widget=forms.HiddenInput, required=False)
    internal_reference = forms.CharField(
        label=_(u"Other reference"), max_length=60, required=False)
    name = forms.CharField(label=_(u"Name"), required=False, max_length=100)
    creation_date = forms.DateField(label=_(u"Creation date"),
                                    initial=get_now, widget=widgets.JQueryDate)
    file_type = forms.ChoiceField(label=_("File type"), choices=[])
    related_file = forms.IntegerField(
        label=_("Related file"), required=False,
        widget=widgets.JQueryAutoComplete(reverse_lazy('autocomplete-file'),
                                          associated_model=models.File),
        validators=[valid_id(models.File)])
    comment = forms.CharField(label=_(u"Comment"), widget=forms.Textarea,
                              required=False)
    total_surface = forms.FloatField(
        required=False, widget=widgets.AreaWidget,
        label=_(u"Total surface (m2)"),
        validators=[validators.MinValueValidator(0),
                    validators.MaxValueValidator(999999999)])
    address = forms.CharField(label=_(u"Main address"), widget=forms.Textarea)
    address_complement = forms.CharField(label=_(u"Main address - complement"),
                                         required=False)

    def __init__(self, *args, **kwargs):
        super(FileFormGeneral, self).__init__(*args, **kwargs)
        self.fields['file_type'].choices = models.FileType.get_types(
            initial=self.init_data.get('file_type'))
        self.fields['file_type'].help_text = models.FileType.get_help()
        q = models.File.objects\
                  .filter(internal_reference__isnull=False)\
                  .exclude(internal_reference='').order_by('-pk')
        if q.count():
            lbl = self.fields['internal_reference'].label
            lbl += _(u"<br/>(last recorded: %s)") % (
                q.all()[0].internal_reference)
            self.fields['internal_reference'].label = mark_safe(lbl)


class FileFormGeneralRO(FileFormGeneral):
    year = forms.IntegerField(
        label=_(u"Year"), widget=forms.TextInput(attrs={'readonly': True}))
    numeric_reference = forms.IntegerField(
        label=_(u"Numeric reference"), widget=forms.TextInput())
    id = forms.IntegerField(' ', widget=forms.HiddenInput, required=False)

    def clean(self):
        cleaned_data = self.cleaned_data
        year = cleaned_data.get('year')
        pk = cleaned_data.get('id')
        numeric_reference = cleaned_data.get('numeric_reference')
        q = models.File.objects\
                       .filter(year=year, numeric_reference=numeric_reference)\
                       .exclude(pk=pk)
        if numeric_reference and q.count():
            raise forms.ValidationError(
                _(u"Another file with this numeric id exists."))
        return cleaned_data

RESPONSIBLE_PLANNING_SERVICE, created = PersonType.objects.get_or_create(
    txt_idx='responsible_planning_service')

RESPONSIBLE_PLANNING_SERVICE_ORGA, created = \
    OrganizationType.objects.get_or_create(txt_idx='planning_service')


class FileFormPreventive(ManageOldType, forms.Form):
    form_label = _(u"Preventive informations")
    associated_models = {'general_contractor': Person,
                         'saisine_type': models.SaisineType,
                         'permit_type': models.PermitType,
                         'responsible_town_planning_service': Person}
    general_contractor = forms.IntegerField(
        label=_(u"General contractor"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy(
                'autocomplete-person',
                args=[PersonType.objects.get(txt_idx='general_contractor').pk]
            ),
            limit={'person_types': [
                PersonType.objects.get(txt_idx='general_contractor').pk]},
            associated_model=Person, new=True),
        validators=[valid_id(Person)])
    responsible_town_planning_service = forms.IntegerField(
        required=False,
        label=_(u"Responsible for town planning service"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy(
                'autocomplete-person',
                args=[RESPONSIBLE_PLANNING_SERVICE.pk]
            ),
            limit={'person_types': [
                RESPONSIBLE_PLANNING_SERVICE.pk]},
            associated_model=Person, new=True),
        validators=[valid_id(Person)])
    permit_type = forms.ChoiceField(label=_(u"Permit type"), required=False,
                                    choices=[])
    permit_reference = forms.CharField(
        label=_(u"Permit reference"), required=False,
        validators=[validators.MaxLengthValidator(60)])
    total_developed_surface = forms.FloatField(
        widget=widgets.AreaWidget, label=_(u"Total developed surface (m2)"),
        required=False, validators=[validators.MinValueValidator(0),
                                    validators.MaxValueValidator(999999999)])
    if settings.COUNTRY == 'fr':
        saisine_type = forms.ChoiceField(label=_(u"Saisine type"),
                                         choices=[])
    reception_date = forms.DateField(
        label=_(u"Reception date"), initial=get_now, widget=widgets.JQueryDate)

    def __init__(self, *args, **kwargs):
        super(FileFormPreventive, self).__init__(*args, **kwargs)
        if 'saisine_type' in self.fields:
            self.fields['saisine_type'].choices = \
                models.SaisineType.get_types(
                    initial=self.init_data.get('saisine_type'))
            self.fields['saisine_type'].help_text = \
                models.SaisineType.get_help()
        self.fields['permit_type'].choices = models.PermitType.get_types(
            initial=self.init_data.get('permit_type'), default='NP')
        self.fields['permit_type'].help_text = models.PermitType.get_help()


class FileFormResearch(ManageOldType, forms.Form):
    form_label = _("Research archaeology")
    base_model = 'department'
    associated_models = {'scientist': Person,
                         'requested_operation_type': OperationType,
                         'organization': Organization,
                         'department': Department}
    department = widgets.MultipleAutocompleteField(
        model=Department,
        label=_("Departments"),
        required=False)
    scientist = forms.IntegerField(
        widget=widgets.JQueryAutoComplete(
            reverse_lazy(
                'autocomplete-person',
                args=["_".join([unicode(HEAD_SCIENTIST.pk),
                                unicode(SRA_AGENT.pk)])]),
                limit={'person_types': [unicode(HEAD_SCIENTIST.pk),
                                        unicode(SRA_AGENT.pk)]},
            associated_model=Person, new=True),
        label=_(u"Scientist in charge"))
    requested_operation_type = forms.ChoiceField(
        label=_(u"Requested operation type"), choices=[])
    organization = forms.IntegerField(
        label=_(u"Lead organization"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-organization'),
            associated_model=Organization, new=True),
        validators=[valid_id(Organization)], required=False)
    if settings.COUNTRY == 'fr':
        cira_advised = forms.NullBooleanField(label=u"Passage en CIRA",
                                              required=False)
    research_comment = forms.CharField(
        label=_(u"Comment"), widget=forms.Textarea, required=False)
    if settings.COUNTRY == 'fr':
        mh_register = forms.NullBooleanField(
            label=u"Sur Monument Historique classé", required=False)
        mh_listing = forms.NullBooleanField(
            label=u"Sur Monument Historique inscrit", required=False)
    classified_area = forms.NullBooleanField(
        label=_(u"Classified area"), required=False)
    protected_area = forms.NullBooleanField(
        label=_(u"Protected area"), required=False)

    def __init__(self, *args, **kwargs):
        super(FileFormResearch, self).__init__(*args, **kwargs)
        self.fields['requested_operation_type'].choices = \
            OperationType.get_types(
                dct={"preventive": False},
                initial=self.init_data.get('requested_operation_type'))
        self.fields['requested_operation_type'].help_text = \
            OperationType.get_help()


class FinalFileClosingForm(FinalForm):
    confirm_msg = " "
    confirm_end_msg = _(u"Would you like to close this archaeological file?")


class FinalFileDeleteForm(FinalForm):
    confirm_msg = " "
    confirm_end_msg = _(u"Would you like to delete this archaelogical file ?")


class AdministrativeActFileModifySelect(TableSelect):
    year = forms.IntegerField(label=_("Year"))
    index = forms.IntegerField(label=_("Index"))
    if settings.COUNTRY == 'fr':
        ref_sra = forms.CharField(label=u"Référence SRA",
                                  max_length=15)
    act_type = forms.ChoiceField(label=_("Act type"), choices=[])
    act_object = forms.CharField(label=_(u"Object (full text search)"),
                                 max_length=300)
    operation__towns = get_town_field()
    if settings.ISHTAR_DPTS:
        operation__towns__numero_insee__startswith = forms.ChoiceField(
            label=_(u"Department"), choices=[])

    def __init__(self, *args, **kwargs):
        super(AdministrativeActFileModifySelect, self).__init__(*args,
                                                                **kwargs)
        self.fields['act_type'].choices = ActType.get_types(
            dct={'intented_to': 'F'})
        self.fields['act_type'].help_text = ActType.get_help(
            dct={'intented_to': 'F'})
        if settings.ISHTAR_DPTS:
            k = 'operation__towns__numero_insee__startswith'
            self.fields[k].choices = [
                ('', '--')] + list(settings.ISHTAR_DPTS)


class AdministrativeActFileSelect(TableSelect):
    year = forms.IntegerField(label=_("Year"))
    index = forms.IntegerField(label=_("Index"))
    if settings.COUNTRY == 'fr':
        ref_sra = forms.CharField(label=u"Autre référence",
                                  max_length=15)
    act_type = forms.ChoiceField(label=_("Act type"), choices=[])
    indexed = forms.NullBooleanField(label=_(u"Indexed?"))
    associated_file__towns = get_town_field()
    parcel = ParcelField(label=_("Parcel (section/number/public domain)"))
    if settings.ISHTAR_DPTS:
        associated_file__towns__numero_insee__startswith = forms.ChoiceField(
            label=_(u"Department"), choices=[])
    act_object = forms.CharField(label=_(u"Object"),
                                 max_length=300)

    signature_date_after = forms.DateField(
        label=_(u"Signature date after"), widget=widgets.JQueryDate)
    signature_date_before = forms.DateField(
        label=_(u"Signature date before"), widget=widgets.JQueryDate)
    associated_file__name = forms.CharField(
        label=_(u"File name"), max_length=200)
    associated_file__general_contractor = forms.IntegerField(
        label=_(u"General contractor"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy(
                'autocomplete-person',
                args=[
                    PersonType.objects.get(txt_idx='general_contractor').pk]),
                associated_model=Person),
        validators=[valid_id(Person)])
    associated_file__general_contractor__attached_to = forms.IntegerField(
        label=_(u"Organization of general contractor"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy(
                'autocomplete-organization',
                args=[GENERAL_CONTRACTOR_ORGA.pk]),
            associated_model=Organization),
        validators=[valid_id(Organization)])
    associated_file__numeric_reference = forms.IntegerField(
        label=_("File numeric reference"))
    associated_file__year = forms.IntegerField(label=_("File year"))
    associated_file__internal_reference = forms.CharField(
        max_length=200, label=_("File other reference"))
    associated_file__in_charge = forms.IntegerField(
        label=_(u"File in charge"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy(
                'autocomplete-person',
                args=[SRA_AGENT.pk]),
            associated_model=Person),
        validators=[valid_id(Person)])
    associated_file__permit_reference = forms.CharField(
        max_length=200, label=_("File permit reference"))
    history_creator = forms.IntegerField(
        label=_(u"Created by"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy(
                'autocomplete-person',
                args=['0', 'user']),
            associated_model=Person),
        validators=[valid_id(Person)])
    history_modifier = forms.IntegerField(
        label=_(u"Modified by"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-person',
                         args=['0', 'user']),
            associated_model=Person),
        validators=[valid_id(Person)])

    def __init__(self, *args, **kwargs):
        super(AdministrativeActFileSelect, self).__init__(*args, **kwargs)
        self.fields['act_type'].choices = ActType.get_types(
            dct={'intented_to': 'F'})
        self.fields['act_type'].help_text = ActType.get_help(
            dct={'intented_to': 'F'})
        if settings.ISHTAR_DPTS:
            k = 'associated_file__towns__numero_insee__startswith'
            self.fields[k].choices = [
                ('', '--')] + list(settings.ISHTAR_DPTS)

    def get_input_ids(self):
        ids = super(AdministrativeActFileSelect, self).get_input_ids()
        ids.pop(ids.index('parcel'))
        ids.append('parcel_0')
        ids.append('parcel_1')
        ids.append('parcel_2')
        return ids


class AdministrativeActFileFormSelection(AdministrativeActOpeFormSelection):
    pk = forms.IntegerField(
        label="", required=False,
        widget=widgets.JQueryJqGrid(
            reverse_lazy('get-administrativeactfile'),
            AdministrativeActFileSelect, AdministrativeAct,
            table_cols='TABLE_COLS_FILE'),
        validators=[valid_id(AdministrativeAct)])


class AdministrativeActFileModifyFormSelection(
        AdministrativeActOpeFormSelection):
    pk = forms.IntegerField(
        label="", required=False,
        widget=widgets.JQueryJqGrid(
            reverse_lazy('get-administrativeactfile'),
            AdministrativeActFileModifySelect, AdministrativeAct,
            table_cols='TABLE_COLS_FILE'),
        validators=[valid_id(AdministrativeAct)])


class AdministrativeActFileForm(AdministrativeActOpeForm):
    act_type = forms.ChoiceField(label=_(u"Act type"), choices=[])

    def __init__(self, *args, **kwargs):
        super(AdministrativeActFileForm, self).__init__(*args, **kwargs)
        self.fields['act_type'].choices = ActType.get_types(
            initial=self.init_data.get('act_type'),
            dct={'intented_to': 'F'})
        self.fields['act_type'].help_text = ActType.get_help(
            dct={'intented_to': 'F'})


class AdministrativeActFileModifForm(AdministrativeActModifForm,
                                     AdministrativeActFileForm):
    pk = forms.IntegerField(required=False, widget=forms.HiddenInput)
    index = forms.IntegerField(label=_("Index"), required=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from ishtar_common.data_importer import *

from archaeological_context_records import models


class ContextRecordsImporterBibracte(Importer):
    DESC = u"Exports Bibracte : importeur pour l'onglet UE"
    OBJECT_CLS = models.ContextRecord
    DEFAULTS = {}
    LINE_FORMAT = [
        # ID operation
        ImportFormater('operation__operation_code', IntegerFormater(),
                       duplicate_fields=[('parcel__operation__operation_code',
                                          False)]),
        # ID UE
        ImportFormater('external_id', UnicodeFormater(120),
                       duplicate_fields=[('label', False)],),
        # Type
        ImportFormater('unit', TypeFormater(models.Unit), required=False),
        # description
        ImportFormater('description', UnicodeFormater(1000), required=False,),
        # interprétation
        ImportFormater('interpretation', UnicodeFormater(1000),
                       required=False,),
        # date ouverture
        ImportFormater('opening_date', DateFormater(['%Y/%m/%d']),
                       required=False,),
        # date fermeture
        ImportFormater('closing_date', DateFormater(['%Y/%m/%d']),
                       required=False,),
        # lien vers parcelle
        ImportFormater('parcel__external_id', UnicodeFormater(12),
                       required=False,),
        # lien vers ID sig
        None,
        # commentaire
        ImportFormater('comment', UnicodeFormater(1000), required=False,),
        # ????
        None,
        # chrono #TODO! pas de vrai création de nouvelle et en cas de modif
        # c'est la zone
        ImportFormater('datings__period', TypeFormater(models.Period),
                       required=False),
    ]


class ContextRecordsRelationImporterBibracte(Importer):
    DESC = u"Exports Bibracte : importeur pour l'onglet relations entre UE"
    OBJECT_CLS = models.RecordRelations
    DEFAULTS = {}
    LINE_FORMAT = [
        # code OA
        ImportFormater(
            'left_record__operation__operation_code', IntegerFormater(),
            duplicate_fields=[('right_record__operation__operation_code',
                               False)],),
        # identifiant UE 1
        ImportFormater('left_record__external_id', UnicodeFormater(120),),
        # type relation
        ImportFormater('relation_type', TypeFormater(models.RelationType),),
        # identifiant UE 2
        ImportFormater('right_record__external_id', UnicodeFormater(120),),
    ]



# -*- coding: utf-8 -*-
import os
from setuptools import setup  # , find_packages
import version

try:
    reqs = open(os.path.join(os.path.dirname(__file__),
                             'requirements.txt')).read()
except (IOError, OSError):
    reqs = ''

setup(
    name='ishtar',
    version=version.get_version(),
    description="Ishtar is a database to manage the finds and "
                "documentation from archaeological operations.",
    long_description=open('README.rst').read(),
    author=u'Étienne Loks',
    author_email='etienne.loks@iggdrasil.net',
    url='http://ishtar-archeo.net/',
    license='AGPL v3 licence, see COPYING',
    packages=[
        'archaeological_finds', 'example_project',
        'archaeological_warehouse', 'archaeological_files_pdl',
        'archaeological_files', 'xhtml2odt', 'simple_history', 'ishtar_common',
        'archaeological_operations', 'archaeological_context_records',
        'ishtar_pdl'],
    include_package_data=True,
    install_requires=reqs,
    # test_suite = "",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf.urls.defaults import *

# from ishtar_common.wizards import check_rights
import views

# be carreful: each check_rights must be relevant with ishtar_menu

# forms
urlpatterns = patterns(
    '',
    url(r'warehouse_packaging/(?P<step>.+)?$',
        views.warehouse_packaging_wizard, name='warehouse_packaging'),
)

urlpatterns += patterns(
    'archaeological_warehouse.views',
    url(r'new-warehouse/(?P<parent_name>.+)?/$',
        'new_warehouse', name='new-warehouse'),
    url(r'autocomplete-warehouse/$', 'autocomplete_warehouse',
        name='autocomplete-warehouse'),
    url(r'new-container/(?P<parent_name>.+)?/$',
        'new_container', name='new-container'),
    url(r'get-container/$', 'get_container',
        name='get-container'),
    url(r'autocomplete-container/?$',
        'autocomplete_container', name='autocomplete-container'),
)



# from https://djangosnippets.org/snippets/2283/

from django.db import transaction
from django.db.models import get_models, Model
from django.contrib.contenttypes.generic import GenericForeignKey


@transaction.commit_on_success
def merge_model_objects(primary_object, alias_objects=[], keep_old=False):
    """
    Use this function to merge model objects (i.e. Users, Organizations, Polls,
    etc.) and migrate all of the related fields from the alias objects to the
    primary object.

    Usage:
    from django.contrib.auth.models import User
    primary_user = User.objects.get(email='good_email@example.com')
    duplicate_user = User.objects.get(email='good_email+duplicate@example.com')
    merge_model_objects(primary_user, duplicate_user)
    """
    MERGE_FIELDS = ('merge_candidate', 'merge_exclusion')

    if not isinstance(alias_objects, list):
        alias_objects = [alias_objects]

    # check that all aliases are the same class as primary one and that
    # they are subclass of model
    primary_class = primary_object.__class__

    if not issubclass(primary_class, Model):
        raise TypeError('Only django.db.models.Model subclasses can be merged')

    for alias_object in alias_objects:
        if not isinstance(alias_object, primary_class):
            raise TypeError('Only models of same class can be merged')

    # Get a list of all GenericForeignKeys in all models
    # TODO: this is a bit of a hack, since the generics framework should
    # provide a similar
    # method to the ForeignKey field for accessing the generic related fields.
    generic_fields = []
    for model in get_models():
        for field_name, field in filter(lambda x: isinstance(
                x[1], GenericForeignKey), model.__dict__.iteritems()):
            generic_fields.append(field)

    blank_local_fields = set()
    for field in primary_object._meta.local_fields:
        value = getattr(primary_object, field.attname)
        # string fields with only spaces are empty fields
        if isinstance(value, unicode) or isinstance(value, str):
            value = value.strip()
        if value in [None, '']:
            blank_local_fields.add(field.attname)

    # Loop through all alias objects and migrate their data to the primary
    # object.
    for alias_object in alias_objects:
        # Migrate all foreign key references from alias object to primary
        # object.
        for related_object in alias_object._meta.get_all_related_objects():
            # The variable name on the alias_object model.
            alias_varname = related_object.get_accessor_name()
            # The variable name on the related model.
            obj_varname = related_object.field.name
            related_objects = getattr(alias_object, alias_varname)
            for obj in related_objects.all():
                setattr(obj, obj_varname, primary_object)
                obj.save()

        # Migrate all many to many references from alias object to primary
        # object.
        related_many_objects = \
            alias_object._meta.get_all_related_many_to_many_objects()
        related_many_object_names = set()
        for related_many_object in related_many_objects:
            alias_varname = related_many_object.get_accessor_name()
            obj_varname = related_many_object.field.name
            if alias_varname in MERGE_FIELDS or obj_varname in MERGE_FIELDS:
                continue

            if alias_varname is not None:
                # standard case
                related_many_objects = getattr(
                    alias_object, alias_varname).all()
                related_many_object_names.add(alias_varname)
            else:
                # special case, symmetrical relation, no reverse accessor
                related_many_objects = getattr(alias_object, obj_varname).all()
                related_many_object_names.add(obj_varname)
            for obj in related_many_objects.all():
                getattr(obj, obj_varname).remove(alias_object)
                getattr(obj, obj_varname).add(primary_object)

        # Migrate local many to many references from alias object to primary
        # object.
        for many_to_many_object in alias_object._meta.many_to_many:
            alias_varname = many_to_many_object.get_attname()
            if alias_varname in related_many_object_names or \
               alias_varname in MERGE_FIELDS:
                continue

            many_to_many_objects = getattr(alias_object, alias_varname).all()
            if alias_varname in blank_local_fields:
                blank_local_fields.pop(alias_varname)
            for obj in many_to_many_objects.all():
                getattr(alias_object, alias_varname).remove(obj)
                getattr(primary_object, alias_varname).add(obj)

        # Migrate all generic foreign key references from alias object to
        # primary object.
        for field in generic_fields:
            filter_kwargs = {}
            filter_kwargs[field.fk_field] = alias_object._get_pk_val()
            filter_kwargs[field.ct_field] = field.get_content_type(
                alias_object)
            for generic_related_object in field.model.objects.filter(
                    **filter_kwargs):
                setattr(generic_related_object, field.name, primary_object)
                generic_related_object.save()

        # Try to fill all missing values in primary object by values of
        # duplicates
        filled_up = set()
        for field_name in blank_local_fields:
            val = getattr(alias_object, field_name)
            if val not in [None, '']:
                setattr(primary_object, field_name, val)
                filled_up.add(field_name)
        blank_local_fields -= filled_up

        if not keep_old:
            alias_object.delete()
    primary_object.save()
    return primary_object



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2015 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Admin description
"""
import csv

from django.conf import settings
from django.contrib import admin
from django.contrib.auth.admin import GroupAdmin
from django.contrib.auth.models import Group
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.defaultfilters import slugify

from django import forms

import models


class ImportGenericForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    csv_file = forms.FileField(
        "CSV file", help_text="Only unicode encoding is managed - convert your"
        " file first")


def gen_import_generic(self, request, queryset):
    form = None

    if 'apply' in request.POST:
        form = ImportGenericForm(request.POST, request.FILES)

        if form.is_valid():
            csv_file = request.FILES['csv_file']
            reader = csv.reader(csv_file)
            idx = 0
            for row in reader:
                slug = slugify(row[0])
                if self.model.objects.filter(txt_idx=slug).count():
                    continue
                obj, c = self.model.objects.get_or_create(
                    label=row[0], txt_idx=slug)
                if c:
                    idx += 1
            self.message_user(request, "Successfully added %d new items." % (
                idx))
            return HttpResponseRedirect(request.get_full_path())
    if not form:
        form = ImportGenericForm(
            initial={'_selected_action':
                     request.POST.getlist(admin.ACTION_CHECKBOX_NAME)})
    return render_to_response(
        'admin/import_from_csv.html', {'csv_form': form},
        context_instance=RequestContext(request))

gen_import_generic.short_description = "Import from a CSV file"


class HistorizedObjectAdmin(admin.ModelAdmin):
    readonly_fields = ('history_modifier',)

    def save_model(self, request, obj, form, change):
        obj.history_modifier = request.user
        obj.save()


class MyGroupAdmin(GroupAdmin):
    class Media:
        css = {
            "all": ("media/admin.css",)
        }

admin.site.unregister(Group)
admin.site.register(Group, MyGroupAdmin)


class IshtarSiteProfileAdmin(admin.ModelAdmin):
    list_display = ('label', 'slug', 'active', 'files', 'context_record',
                    'find', 'warehouse')
    model = models.IshtarSiteProfile

admin.site.register(models.IshtarSiteProfile, IshtarSiteProfileAdmin)


class DepartmentAdmin(admin.ModelAdmin):
    list_display = ('number', 'label',)
    model = models.Department

admin.site.register(models.Department, DepartmentAdmin)


class OrganizationAdmin(HistorizedObjectAdmin):
    list_display = ('pk', 'name', 'organization_type')
    list_filter = ("organization_type",)
    search_fields = ('name',)
    exclude = ('merge_key', 'merge_exclusion', 'merge_candidate', )
    model = models.Organization

admin.site.register(models.Organization, OrganizationAdmin)


class PersonAdmin(HistorizedObjectAdmin):
    list_display = ('pk', 'name', 'surname', 'raw_name', 'email')
    list_filter = ("person_types",)
    search_fields = ('name', 'surname', 'email', 'raw_name')
    exclude = ('merge_key', 'merge_exclusion', 'merge_candidate', )
    model = models.Person

admin.site.register(models.Person, PersonAdmin)


class TownAdmin(admin.ModelAdmin):
    list_display = ['name', ]
    search_fields = ['name']
    if settings.COUNTRY == 'fr':
        list_display += ['numero_insee', 'departement', ]
        search_fields += ['numero_insee', 'departement__label', ]
        list_filter = ("departement",)
    model = models.Town

admin.site.register(models.Town, TownAdmin)


class AuthorAdmin(admin.ModelAdmin):
    list_display = ['person', 'author_type']
    list_filter = ("author_type",)
    model = models.Author

admin.site.register(models.Author, AuthorAdmin)


class PersonTypeAdmin(admin.ModelAdmin):
    model = models.PersonType
    filter_vertical = ('groups',)

admin.site.register(models.PersonType, PersonTypeAdmin)


class GlobalVarAdmin(admin.ModelAdmin):
    list_display = ['slug', 'description', 'value']
admin.site.register(models.GlobalVar, GlobalVarAdmin)


class GeneralTypeAdmin(admin.ModelAdmin):
    list_display = ['label', 'txt_idx', 'available']
    actions = ['import_generic']
    import_generic = gen_import_generic


general_models = [models.OrganizationType, models.SourceType,
                  models.AuthorType]
for model in general_models:
    admin.site.register(model, GeneralTypeAdmin)


class ImporterDefaultValuesInline(admin.TabularInline):
    model = models.ImporterDefaultValues


class ImporterDefaultAdmin(admin.ModelAdmin):
    list_display = ('importer_type', 'target')
    model = models.ImporterDefault
    inlines = (ImporterDefaultValuesInline,)
admin.site.register(models.ImporterDefault, ImporterDefaultAdmin)


class ImporterTypeAdmin(admin.ModelAdmin):
    list_display = ('name', 'associated_models', 'is_template')
admin.site.register(models.ImporterType, ImporterTypeAdmin)


class RegexpAdmin(admin.ModelAdmin):
    list_display = ('name', 'description', "regexp")
admin.site.register(models.Regexp, RegexpAdmin)


class ImporterDuplicateFieldInline(admin.TabularInline):
    model = models.ImporterDuplicateField


class ImportTargetInline(admin.TabularInline):
    model = models.ImportTarget


class ImporterColumnAdmin(admin.ModelAdmin):
    list_display = ('importer_type', 'col_number', 'description',
                    'targets_lbl', 'duplicate_fields_lbl', 'required')
    list_filter = ('importer_type',)
    inlines = (ImporterDuplicateFieldInline, ImportTargetInline)
admin.site.register(models.ImporterColumn, ImporterColumnAdmin)


class FormaterTypeAdmin(admin.ModelAdmin):
    list_display = ('formater_type', 'options')
admin.site.register(models.FormaterType, FormaterTypeAdmin)


class ImportAdmin(admin.ModelAdmin):
    list_display = ('importer_type', 'imported_file', 'user', 'state',
                    'creation_date')
admin.site.register(models.Import, ImportAdmin)


class TargetKeyAdmin(admin.ModelAdmin):
    list_display = ('target', 'importer_type', 'column_nb', 'key',
                    'value', 'is_set')
    list_filter = ("is_set", "target__column__importer_type")
    search_fields = ('target__target', 'value', 'key')
admin.site.register(models.TargetKey, TargetKeyAdmin)


class OperationTypeAdmin(GeneralTypeAdmin):
    list_display = GeneralTypeAdmin.list_display + ['order', 'preventive']
    model = models.OperationType

admin.site.register(models.OperationType, OperationTypeAdmin)


class IshtarUserAdmin(admin.ModelAdmin):
    readonly_fields = ('password',)

admin.site.register(models.IshtarUser, IshtarUserAdmin)

basic_models = [models.DocumentTemplate]
if settings.COUNTRY == 'fr':
    basic_models += [models.Arrondissement, models.Canton]

for model in basic_models:
    admin.site.register(model)
